@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: client.c,v 1.52 2006/04/24 14:59:18 evertonm Exp $ */
+/* $Id: client.c,v 1.55 2006/04/26 10:33:41 evertonm Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -31,6 +31,7 @@ MA 02110-1301 USA
 #include <unistd.h>
 #include <string.h>
 #include <sys/un.h>
+#include <signal.h>
 #include <assert.h>
 
 #include "conf.h"
@@ -356,7 +357,8 @@ static void *on_tcp_duration(oop_source *src, struct timeval tv, void *user)
 
   assert(timercmp(&tv, &session->tv_duration, ==));
 
-  nepim_pipe_stat(stdout, NEPIM_LABEL_TOTAL, pipe->sd, 
+  nepim_pipe_stat(stdout, tv,
+		  NEPIM_LABEL_TOTAL, pipe->sd, 
                   session->byte_total_recv,
                   session->byte_total_sent,
                   session->test_duration,
@@ -383,7 +385,8 @@ static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
 
   assert(timercmp(&tv, &session->tv_interval, ==));
 
-  nepim_pipe_stat(stdout, NEPIM_LABEL_PARTIAL, pipe->sd, 
+  nepim_pipe_stat(stdout, tv,
+		  NEPIM_LABEL_PARTIAL, pipe->sd, 
                   session->byte_interval_recv,
                   session->byte_interval_sent,
                   session->stat_interval,
@@ -583,6 +586,42 @@ static void *on_tcp_keepalive_require(oop_source *src, struct timeval tv, void *
   return OOP_CONTINUE;
 }
 
+static void *on_sigint(oop_source *src, int sig, void *user)
+{
+  nepim_pipe_t *pipe = user;
+
+  assert(sig == SIGINT);
+  assert(pipe);
+
+  fprintf(stderr, "%d: %s %s: interrupted by user\n",
+	  pipe->sd, __FILE__, __PRETTY_FUNCTION__);
+
+  report_broken_pipe_stat(stdout, pipe);
+
+  tcp_pipe_stop(pipe->sd);
+  close(pipe->sd);
+
+  return OOP_CONTINUE;
+}
+
+static void *on_sigterm(oop_source *src, int sig, void *user)
+{
+  nepim_pipe_t *pipe = user;
+
+  assert(sig == SIGTERM);
+  assert(pipe);
+
+  fprintf(stderr, "%d: %s %s: interrupted by user\n",
+	  pipe->sd, __FILE__, __PRETTY_FUNCTION__);
+
+  report_broken_pipe_stat(stdout, pipe);
+
+  tcp_pipe_stop(pipe->sd);
+  close(pipe->sd);
+
+  return OOP_CONTINUE;
+}
+
 static void tcp_pipe_start(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
@@ -639,6 +678,11 @@ static void tcp_pipe_start(int sd)
 
   if (session->keepalive_require)
     nepim_pipe_schedule_keepalive_timer(pipe, on_tcp_keepalive_require);
+
+  nepim_global.oop_src->on_signal(nepim_global.oop_src,
+				  SIGINT, on_sigint, pipe);
+  nepim_global.oop_src->on_signal(nepim_global.oop_src,
+				  SIGTERM, on_sigterm, pipe);
 }
 
 static void *on_tcp_pipe_start(oop_source *src, struct timeval tv, void *user)
@@ -646,6 +690,7 @@ static void *on_tcp_pipe_start(oop_source *src, struct timeval tv, void *user)
   nepim_pipe_t *pipe = user;
 
   tcp_pipe_start(pipe->sd);
+
   return OOP_CONTINUE;
 }
 
@@ -673,6 +718,11 @@ static void tcp_pipe_stop(int sd)
   nepim_session_t *session;
   assert(pipe);
 
+  nepim_global.oop_src->cancel_signal(nepim_global.oop_src,
+				      SIGINT, on_sigint, pipe);
+  nepim_global.oop_src->cancel_signal(nepim_global.oop_src,
+				      SIGTERM, on_sigterm, pipe);
+
   session = &pipe->session;
 
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
@@ -961,8 +1011,6 @@ void nepim_udp_clients(const char *host_list);
 
 void nepim_client_run()
 {
-  void *result;
-
   nepim_pipe_set_init(&pipes);
 
   if (nepim_global.udp_mode) {
@@ -974,5 +1022,23 @@ void nepim_client_run()
   else
     spawn_tcp_clients(nepim_global.hostname);
 
-  result = oop_sys_run(nepim_global.oop_sys);
+  for (;;) {
+    void *result = oop_sys_run(nepim_global.oop_sys);
+    if (result == OOP_ERROR) {
+      fprintf(stderr, "%s: event system source error\n",
+              nepim_global.prog_name);
+      continue;
+    }
+    if (result == OOP_CONTINUE) {
+      fprintf(stderr, "%s: no event sink registered\n",
+              nepim_global.prog_name);
+      break;
+    }
+    if (result == OOP_HALT) {
+      fprintf(stderr, "%s: some event sink requested termination\n",
+              nepim_global.prog_name);
+      break;
+    }
+    assert(0);
+  }
 }
