@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: common.c,v 1.50 2006/04/20 14:01:01 evertonm Exp $ */
+/* $Id: common.c,v 1.57 2006/05/11 12:40:57 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,10 +41,10 @@ const char * const NEPIM_LABEL_TOTAL_BROKEN = "avg(broken)";
 
 const char *nepim_version()
 {
-  return "0.22";
+  return "0.23";
 }
 
-static void nepim_dump_stat(FILE *out, const char *label,
+static void nepim_dump_stat(FILE *out, struct timeval now, const char *label,
                             long long bytes_recv, long long bytes_sent, 
                             float interval, long sec_start, 
                             long sec_duration, int reads, int writes)
@@ -53,12 +53,7 @@ static void nepim_dump_stat(FILE *out, const char *label,
   float kbps_sent = bytes_sent / (interval * 125); /* 8/1000 = 1/125 */
   float read_rate = reads / interval;
   float write_rate = writes / interval;
-  struct timeval now;
-  int result;
-      
-  result = gettimeofday(&now, 0);
-  assert(!result);
-
+  
   fprintf(out,
           ": %s %ld/%ld kbps_in=%.2f kbps_out=%.2f rcv/s=%.2f snd/s=%.2f", 
           label,
@@ -70,14 +65,15 @@ static void nepim_dump_stat(FILE *out, const char *label,
           write_rate);
 }
 
-void nepim_pipe_stat(FILE *out, const char *label, int sd,
+void nepim_pipe_stat(FILE *out, struct timeval now,
+		     const char *label, int sd,
                      long long bytes_recv, long long bytes_sent, 
                      float interval, long sec_start, 
                      long sec_duration, int reads, int writes)
 {
   fprintf(out, "%d", sd);
 
-  nepim_dump_stat(out, label, bytes_recv, bytes_sent, 
+  nepim_dump_stat(out, now, label, bytes_recv, bytes_sent, 
                   interval, sec_start, sec_duration,
                   reads, writes);
 
@@ -87,29 +83,32 @@ void nepim_pipe_stat(FILE *out, const char *label, int sd,
   fflush(out);
 }
 
-void nepim_slot_stat(FILE *out, const char *label, int sd,
+void nepim_slot_stat(FILE *out, struct timeval now,
+		     const char *label, int sd,
                      int local_slot, int remote_slot,
                      long long bytes_recv, long long bytes_sent, 
                      float interval, long sec_start, 
                      long sec_duration, int reads, int writes,
-                     int pkt_lost, int pkt_dup)
+                     uint64_t pkt_expected, uint64_t pkt_lost, uint64_t pkt_ooo)
 {
-  fprintf(out, "%d %d-%d", sd, local_slot, remote_slot);
+  fprintf(out, "%d %d-%d",
+	  sd, local_slot, remote_slot);
 
-  nepim_dump_stat(out, label, bytes_recv, bytes_sent, 
+  nepim_dump_stat(out, now, label, bytes_recv, bytes_sent, 
                   interval, sec_start, sec_duration,
                   reads, writes);
 
-  if (nepim_global.udp_exp_stats) {
+  {
     float loss_ratio = 100 * pkt_lost;
-    float dup_ratio = 100 * pkt_dup;
+    float ooo_ratio = 100 * pkt_ooo;
     
-    loss_ratio /= reads;
-    dup_ratio /= reads;
+    loss_ratio /= pkt_expected;
+    ooo_ratio /= pkt_expected;
     
     fprintf(out,
-            " lost=%d/%.2f%% dup=%d/%.2f%%",
-            pkt_lost, loss_ratio, pkt_dup, dup_ratio);
+            " loss=%llu/%.2f%% ooo=%llu/%.2f%%",
+            pkt_lost, loss_ratio,
+	    pkt_ooo, ooo_ratio);
   }
 
   fprintf(out, "\n");
@@ -118,24 +117,21 @@ void nepim_slot_stat(FILE *out, const char *label, int sd,
   fflush(out);
 }
 
-void report_broken_pipe_stat(FILE *out, const nepim_pipe_t *pipe)
+void report_broken_pipe_stat_at(FILE *out, const nepim_pipe_t *pipe,
+				struct timeval now)
 {
-  struct timeval now;
-  int result;
+  const nepim_session_t *session = &pipe->session;
   float elapsed_sec;
   float elapsed_usec;
   float elapsed;
-  const nepim_session_t *session = &pipe->session;
       
-  result = gettimeofday(&now, 0);
-  assert(!result);
-
   elapsed_sec = now.tv_sec - session->tv_start.tv_sec;
   elapsed_usec = now.tv_usec - session->tv_start.tv_usec;
 
   elapsed = elapsed_sec + elapsed_usec / 1000000;
 
   nepim_pipe_stat(out, 
+		  now,
                   NEPIM_LABEL_TOTAL_BROKEN,
                   pipe->sd, 
                   session->byte_total_recv,
@@ -147,24 +143,35 @@ void report_broken_pipe_stat(FILE *out, const nepim_pipe_t *pipe)
                   session->total_writes);
 }
 
-void report_broken_slot_stat(FILE *out, const nepim_slot_t *slot)
+void report_broken_pipe_stat(FILE *out, const nepim_pipe_t *pipe)
 {
   struct timeval now;
   int result;
-  float elapsed_sec;
-  float elapsed_usec;
-  float elapsed;
-  const nepim_session_t *session = &slot->session;
       
   result = gettimeofday(&now, 0);
   assert(!result);
 
+  report_broken_pipe_stat_at(out, pipe, now);
+}
+
+void report_broken_slot_stat_at(FILE *out, nepim_slot_t *slot,
+				struct timeval now)
+{
+  const nepim_session_t *session = &slot->session;
+  float elapsed_sec;
+  float elapsed_usec;
+  float elapsed;
+
+  assert(!slot->session.duration_done);
+
+  nepim_slot_update_pkt_stat(slot);
+  
   elapsed_sec = now.tv_sec - session->tv_start.tv_sec;
   elapsed_usec = now.tv_usec - session->tv_start.tv_usec;
-
   elapsed = elapsed_sec + elapsed_usec / 1000000;
 
-  nepim_slot_stat(out, 
+  nepim_slot_stat(out,
+		  now,
                   NEPIM_LABEL_TOTAL_BROKEN,
                   slot->udp_sd,
                   slot->index, 
@@ -176,8 +183,20 @@ void report_broken_slot_stat(FILE *out, const nepim_slot_t *slot)
                   session->test_duration,
                   session->total_reads,
                   session->total_writes,
+		  slot->seq_highest_recv,
                   slot->total_pkt_lost,
-                  slot->total_pkt_dup);
+                  slot->total_pkt_ooo);
+}
+
+void report_broken_slot_stat(FILE *out, nepim_slot_t *slot)
+{
+  struct timeval now;
+  int result;
+      
+  result = gettimeofday(&now, 0);
+  assert(!result);
+
+  report_broken_slot_stat_at(out, slot, now);
 }
 
 #define NEPIM_MEGA (1000000)
