@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: common.h,v 1.19 2005/12/20 09:36:09 evertonm Exp $ */
+/* $Id: common.h,v 1.23 2006/05/11 00:29:02 evertonm Exp $ */
 
 #ifndef NEPIM_COMMON_H
 #define NEPIM_COMMON_H
@@ -42,20 +42,27 @@ extern const char * const NEPIM_LABEL_TOTAL;
 
 const char *nepim_version();
 
-void nepim_pipe_stat(FILE *out, const char *label, int sd,
+void nepim_pipe_stat(FILE *out, struct timeval now,
+		     const char *label, int sd,
                      long long bytes_recv, long long bytes_sent, 
                      float interval, long sec_start, 
                      long sec_duration, int reads, int writes);
 
-void nepim_slot_stat(FILE *out, const char *label, int sd,
+void nepim_slot_stat(FILE *out, struct timeval now,
+		     const char *label, int sd,
                      int local_slot, int remote_slot,
                      long long bytes_recv, long long bytes_sent, 
                      float interval, long sec_start, 
                      long sec_duration, int reads, int writes,
-                     int pkt_lost, int pkt_dup);
+                     uint64_t pkt_expected, uint64_t pkt_lost, uint64_t pkt_ooo);
 
 void report_broken_pipe_stat(FILE *out, const nepim_pipe_t *pipe);
-void report_broken_slot_stat(FILE *out, const nepim_slot_t *slot);
+void report_broken_slot_stat(FILE *out, nepim_slot_t *slot);
+
+void report_broken_pipe_stat_at(FILE *out, const nepim_pipe_t *pipe,
+				struct timeval now);
+void report_broken_slot_stat_at(FILE *out, nepim_slot_t *slot,
+				struct timeval now);
 
 void nepim_timer_usec_add(struct timeval *tv, susec_t usec);
 
