@@ -71,9 +71,6 @@ struct nepim_global_t {
   unsigned char fill_byte;             /* client */
   int           pause_duration;
   const char   *password;
-  int           udp_exp_stats;           /* boolean */
-  int           udp_exp_loss;
-  int           udp_exp_dup;
   const char   *tcpwrap;
   int           search_unix_socket; /* boolean */
   int           soft_error_verbose; /* boolean */
