@@ -21,7 +21,7 @@ MA 02110-1301 USA
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.47 2006/04/24 15:06:21 evertonm Exp $
+  $Id: main.c,v 1.50 2006/05/11 12:40:57 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -103,9 +103,6 @@ static void usage(FILE *out, const char *prog_name)
 	  "  -q             quietly hide soft read/write errors\n"
 	  "  -y tos         ip tos (defaults to %d, tos=-1 means don't change)\n"
 	  "  -K udp-ka-time UDP keepalive timeout in ms (defaults to %ld ms)\n"
-          "  -x [win]       experimental UDP reception statistics (win=%d)\n"
-          "  -L loss        experimental UDP sending loss\n"
-          "  -D dup         experimental UDP sending duplication\n"
           "client-only options:\n"
           "  -u             UDP mode (defaults to TCP)\n"
           "  -s             client-send simplex mode (defaults to server-send)\n"
@@ -160,7 +157,6 @@ static void usage(FILE *out, const char *prog_name)
           nepim_global.udp_read_size,
 	  nepim_global.tos,
 	  (long) nepim_global.udp_keepalive_recv_timer,
-          nepim_global.udp_win_max,
 	  UDP_OVERHEAD,
 	  TCP_OVERHEAD,
           nepim_global.pipes,
@@ -314,41 +310,6 @@ int main(int argc, const char *argv[])
       continue;
     }
 
-    if (!strcmp(arg, "-L")) {
-      ++i;
-      if (i >= argc) {
-        fprintf(stderr, "%s: missing UDP loss\n",
-                nepim_global.prog_name);
-        exit(1);
-      }
-      nepim_global.udp_exp_loss = atoi(argv[i]);
-      continue;
-    }
-
-    if (!strcmp(arg, "-D")) {
-      ++i;
-      if (i >= argc) {
-        fprintf(stderr, "%s: missing UDP duplication\n",
-                nepim_global.prog_name);
-        exit(1);
-      }
-      nepim_global.udp_exp_dup = atoi(argv[i]);
-      continue;
-    }
-
-    if (!strcmp(arg, "-x")) {
-      int j = i + 1;
-      nepim_global.udp_exp_stats = 1;
-
-      if (j < argc)
-        if (*argv[j] != '-') {
-          ++i;
-          nepim_global.udp_win_max = atoi(argv[i]);
-        }
-
-      continue;
-    }
-
     if (!strcmp(arg, "-4")) {
       nepim_global.no_inet4 = 1;
       continue;
