@@ -18,14 +18,13 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: server.c,v 1.63 2006/04/24 14:59:18 evertonm Exp $ */
+/* $Id: server.c,v 1.66 2006/04/26 10:33:42 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
-#include <assert.h>
 #include <string.h>
 #include <errno.h>
 #include <netinet/in.h>
@@ -33,6 +32,8 @@ MA 02110-1301 USA
 #include <sys/un.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <signal.h>
+#include <assert.h>
 
 #include "conf.h"
 #include "sock.h"
@@ -50,6 +51,8 @@ nepim_usock_set_t udp_tab; /* udp socket table */
 static const char * const INET_ANY = "0.0.0.0";
 static const char * const INET6_ANY = "::";
 
+void nepim_slot_kill(nepim_slot_t *slot); /* udp_server.c */
+
 static void schedule_stat_interval(int sd);
 static void tcp_pipe_cancel_timers(int sd);
 static void tcp_pipe_cancel_io(int sd);
@@ -338,7 +341,10 @@ static void *on_tcp_duration(oop_source *src, struct timeval tv, void *user)
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
 
-  nepim_pipe_stat(stdout, NEPIM_LABEL_TOTAL, pipe->sd, 
+  assert(timercmp(&tv, &session->tv_duration, ==));
+
+  nepim_pipe_stat(stdout, tv,
+		  NEPIM_LABEL_TOTAL, pipe->sd, 
                   session->byte_total_recv,
                   session->byte_total_sent,
                   session->test_duration,
@@ -363,7 +369,10 @@ static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
 
-  nepim_pipe_stat(stdout, NEPIM_LABEL_PARTIAL, pipe->sd, 
+  assert(timercmp(&tv, &session->tv_interval, ==));
+
+  nepim_pipe_stat(stdout, tv,
+		  NEPIM_LABEL_PARTIAL, pipe->sd, 
                   session->byte_interval_recv,
                   session->byte_interval_sent,
                   session->stat_interval,
@@ -1120,9 +1129,64 @@ static int parse_unix_dgram_listeners(const char *list)
   return listeners;
 }
 
+static void break_all()
+{
+  struct timeval now;
+  int result;
+  int i;
+      
+  result = gettimeofday(&now, 0);
+  assert(!result);
+
+  for (i = 0; i < pipes.array.capacity; ++i) {
+    nepim_pipe_t *pipe = pipes.array.head[i];
+    if (pipe) {
+      if (!pipe->session.duration_done)
+	report_broken_pipe_stat_at(stdout, pipe, now);
+
+      tcp_pipe_kill(pipe->sd);
+    }
+  }
+
+  for (i = 0; i < slots.array.capacity; ++i) {
+    nepim_slot_t *slot = slots.array.head[i];
+    if (slot) {
+      if (!slot->session.duration_done)
+	report_broken_slot_stat_at(stdout, slot, now);
+
+      nepim_slot_kill(slot);
+    }
+  }
+}
+
+static void *on_sigint(oop_source *src, int sig, void *user)
+{
+  assert(sig == SIGINT);
+  assert(!user);
+
+  fprintf(stderr, "%s %s: interrupted by user\n",
+	  __FILE__, __PRETTY_FUNCTION__);
+
+  break_all();
+
+  return OOP_HALT;
+}
+
+static void *on_sigterm(oop_source *src, int sig, void *user)
+{
+  assert(sig == SIGTERM);
+  assert(!user);
+
+  fprintf(stderr, "%s %s: interrupted by user\n",
+	  __FILE__, __PRETTY_FUNCTION__);
+
+  break_all();
+
+  return OOP_HALT;
+}
+
 void nepim_server_run()
 {
-  void *result;
   int tcp_listeners = 0;
   int udp_listeners = 0;
   int unix_listeners;
@@ -1163,6 +1227,11 @@ void nepim_server_run()
     udp_listeners += nepim_udp_listener(INET_ANY, nepim_global.portname,
                                         0 /* no join */);
   }
+
+  nepim_global.oop_src->on_signal(nepim_global.oop_src,
+				  SIGINT, on_sigint, 0);
+  nepim_global.oop_src->on_signal(nepim_global.oop_src,
+				  SIGTERM, on_sigterm, 0);
   
   if (tcp_listeners ||
       udp_listeners ||
@@ -1170,7 +1239,30 @@ void nepim_server_run()
       unix_dgram_listeners)
     fprintf(stderr, "%s: server ready\n", nepim_global.prog_name);
 
-  result = oop_sys_run(nepim_global.oop_sys);
+  for (;;) {
+    void *result = oop_sys_run(nepim_global.oop_sys);
+    if (result == OOP_ERROR) {
+      fprintf(stderr, "%s: event system source error\n",
+	      nepim_global.prog_name);
+      continue;
+    }
+    if (result == OOP_CONTINUE) {
+      fprintf(stderr, "%s: no event sink registered\n",
+	      nepim_global.prog_name);
+      break;
+    }
+    if (result == OOP_HALT) {
+      fprintf(stderr, "%s: some event sink requested termination\n",
+	      nepim_global.prog_name);
+      break;
+    }
+    assert(0);
+  }
+
+  nepim_global.oop_src->cancel_signal(nepim_global.oop_src,
+				      SIGINT, on_sigint, 0);
+  nepim_global.oop_src->cancel_signal(nepim_global.oop_src,
+				      SIGTERM, on_sigterm, 0);
 }
 
 
