@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: sock.c,v 1.33 2006/04/18 23:14:19 evertonm Exp $ */
+/* $Id: sock.c,v 1.34 2006/04/20 14:49:34 evertonm Exp $ */
 
 #include <assert.h>
 #include <netdb.h>
@@ -136,7 +136,7 @@ void nepim_sock_dump_addr(char *buf, int buf_size,
   const char *dst;
   int family;
 
-  if (addr_len < 1) {
+  if (addr_len < 3) {
     assert(buf_size >= 2);
     snprintf(buf, buf_size, "?");
     return;
@@ -289,7 +289,7 @@ static int create_socket(int domain, int type, int protocol, int pmtu_mode, int
     return NEPIM_SOCK_ERR_SOCKET;
 
   if (type == SOCK_STREAM) {
-    result = nepim_socket_tcp_opt(sd, nodelay);
+    result = nepim_socket_tcp_opt(domain, sd, nodelay);
     if (result) {
       close(sd);
       return result;
@@ -537,10 +537,15 @@ int nepim_socket_opt(int sd, int pmtu_mode, int ttl, int tos)
   return NEPIM_SOCK_ERR_NONE;
 }
 
-int nepim_socket_tcp_opt(int sd, int nodelay)
+int nepim_socket_tcp_opt(int family, int sd, int nodelay)
 {
-  if (set_tcp_nodelay(sd, nodelay))
-    return NEPIM_SOCK_ERR_NODELAY;
+  if (set_tcp_nodelay(sd, nodelay)) {
+    /*
+      Tolerate failure in setting NODELAY for PF_UNIX sockets
+    */
+    if (family != PF_UNIX)
+      return NEPIM_SOCK_ERR_NODELAY;
+  }
 
   return NEPIM_SOCK_ERR_NONE;
 }
