@@ -30,7 +30,7 @@ int nepim_socket_ttl(int sd, int ttl);
 int nepim_socket_mcast_ttl(int sd, int mcast_ttl);
 int nepim_socket_pmtu(int sd, int pmtu_mode);
 int nepim_socket_opt(int sd, int pmtu_mode, int ttl, int tos);
-int nepim_socket_tcp_opt(int sd, int nodelay);
+int nepim_socket_tcp_opt(int family, int sd, int nodelay);
 int nepim_socket_keepalive(int sd, int keepalive);
 
 int nepim_socket_pmtu_get_mode(int sd);
