@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_client.c,v 1.57 2006/04/23 06:33:46 evertonm Exp $ */
+/* $Id: udp_client.c,v 1.67 2006/05/11 14:03:17 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -32,6 +32,7 @@ MA 02110-1301 USA
 #include <sys/un.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <signal.h>
 #include <assert.h>
 
 #include "conf.h"
@@ -90,7 +91,8 @@ static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *
             "%d %d-%d: broken slot: peer not sending keepalives?\n",
             slot->udp_sd, slot->index, slot->index_remote);
 
-    report_broken_slot_stat(stdout, slot);
+    if (!session->duration_done)
+      report_broken_slot_stat(stdout, slot);
 
     udp_slot_kill(slot);
 
@@ -127,6 +129,46 @@ static void *on_udp_keepalive_time(oop_source *src, struct timeval tv, void *use
   return OOP_CONTINUE;
 }
 
+static void *on_sigint(oop_source *src, int sig, void *user)
+{
+  nepim_slot_t *slot = user;
+
+  assert(sig == SIGINT);
+  assert(slot);
+
+  fprintf(stderr,
+	  "%d %d-%d: %s %s: interrupted by user\n",
+	  slot->udp_sd, slot->index, slot->index_remote,
+	  __FILE__, __PRETTY_FUNCTION__);
+
+  if (!slot->session.duration_done)
+    report_broken_slot_stat(stdout, slot);
+
+  udp_slot_kill(slot);
+
+  return OOP_CONTINUE;
+}
+
+static void *on_sigterm(oop_source *src, int sig, void *user)
+{
+  nepim_slot_t *slot = user;
+
+  assert(sig == SIGTERM);
+  assert(slot);
+
+  fprintf(stderr,
+	  "%d %d-%d: %s %s: interrupted by user\n",
+	  slot->udp_sd, slot->index, slot->index_remote,
+	  __FILE__, __PRETTY_FUNCTION__);
+
+  if (!slot->session.duration_done)
+    report_broken_slot_stat(stdout, slot);
+
+  udp_slot_kill(slot);
+
+  return OOP_CONTINUE;
+}
+
 static void udp_slot_start(nepim_slot_t *slot)
 {
   nepim_session_t *session;
@@ -175,6 +217,11 @@ static void udp_slot_start(nepim_slot_t *slot)
 
   if (session->keepalive_require)
     nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
+
+  nepim_global.oop_src->on_signal(nepim_global.oop_src,
+				  SIGINT, on_sigint, slot);
+  nepim_global.oop_src->on_signal(nepim_global.oop_src,
+				  SIGTERM, on_sigterm, slot);
 }
 
 static void *on_udp_slot_start(oop_source *src, struct timeval tv, void *user)
@@ -203,18 +250,18 @@ static void schedule_udp_slot_start(nepim_slot_t *slot)
                                 on_udp_slot_start, slot);
 }
 
-static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
+static void udp_consume(nepim_slot_t *slot, size_t len,
+			const nepim_udp_hdr_t *hdr)
 {
   nepim_session_t *session = &slot->session;
 
   /* any valid input traffic accounted as keepalive */
   ++session->keepalives_recv;
 
-  nepim_session_read_add(session, len + session->overhead);
+  if (hdr->type == UDP_TYPE_DATA)
+    nepim_slot_seq_recv(slot, hdr->seq);
 
-  if (nepim_global.udp_exp_stats)
-    nepim_win_add(&slot->win_recv_seq, seq,
-                  &slot->interval_pkt_lost, &slot->interval_pkt_dup);
+  nepim_session_read_add(session, len + session->overhead);
 }
 
 static void *on_udp_read_segment(oop_source *src, int sd,
@@ -267,9 +314,9 @@ static void *on_udp_read_segment(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  if (hdr.version) {
-    fprintf(stderr, "%d: recvfrom: ignoring unknown UDP header version: %d\n",
-            sd, hdr.version);
+  if (hdr.version != UDP_VERSION) {
+    fprintf(stderr, "%d %d-%d: recvfrom: ignoring unknown UDP header version: %d\n",
+            sd, hdr.dst_slot, hdr.src_slot, hdr.version);
     return OOP_CONTINUE;
   }
 
@@ -315,11 +362,10 @@ static void *on_udp_read_segment(oop_source *src, int sd,
       slot->client_writer_status = NEPIM_SLOT_CLIENT_SEND;
     
       schedule_udp_slot_start(slot);
-    
-      return OOP_CONTINUE;
     }
 
     assert(slot->index_remote < 0xFFFF);
+    assert(slot->client_writer_status == NEPIM_SLOT_CLIENT_SEND);
 
     if (slot->index_remote != hdr.src_slot) {
       fprintf(stderr, 
@@ -328,7 +374,7 @@ static void *on_udp_read_segment(oop_source *src, int sd,
       return OOP_CONTINUE;
     }
 
-    udp_consume(slot, rd, hdr.seq);
+    udp_consume(slot, rd, &hdr);
 
     udp_check_packet_data(sd, &hdr, &(slot->session), buf, UDP_HEADER_LEN, rd);
   }
@@ -426,7 +472,12 @@ static void *on_udp_interval(oop_source *src, struct timeval tv, void *user)
   nepim_slot_t *slot = user;
   nepim_session_t *session = &slot->session;
 
-  nepim_slot_stat(stdout, NEPIM_LABEL_PARTIAL, 
+  assert(timercmp(&tv, &session->tv_interval, ==));
+
+  nepim_slot_update_pkt_stat(slot);
+
+  nepim_slot_stat(stdout, tv,
+		  NEPIM_LABEL_PARTIAL, 
                   slot->udp_sd,
                   slot->index,
                   slot->index_remote,
@@ -437,18 +488,20 @@ static void *on_udp_interval(oop_source *src, struct timeval tv, void *user)
                   session->test_duration,
                   session->interval_reads,
                   session->interval_writes,
+		  slot->seq_highest_recv - slot->interval_last_highest_seq,
                   slot->interval_pkt_lost,
-                  slot->interval_pkt_dup);
+                  slot->interval_pkt_ooo);
 
   session->byte_interval_recv = 0;
   session->byte_interval_sent = 0;
   session->interval_reads     = 0;
   session->interval_writes    = 0;
 
-  slot->total_pkt_lost += slot->interval_pkt_lost;
-  slot->total_pkt_dup += slot->interval_pkt_dup;
+  slot->interval_last_highest_seq = slot->seq_highest_recv;
+
+  slot->interval_pkt_recv = 0;
   slot->interval_pkt_lost = 0;
-  slot->interval_pkt_dup = 0;
+  slot->interval_pkt_ooo  = 0;
 
   schedule_stat_interval(slot);
 
@@ -477,10 +530,12 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &slot->session;
   int sd = slot->udp_sd;
 
-  slot->total_pkt_lost += slot->interval_pkt_lost;
-  slot->total_pkt_dup += slot->interval_pkt_dup;
+  assert(timercmp(&tv, &session->tv_duration, ==));
 
-  nepim_slot_stat(stdout, NEPIM_LABEL_TOTAL, 
+  nepim_slot_update_pkt_stat(slot);
+
+  nepim_slot_stat(stdout, tv,
+		  NEPIM_LABEL_TOTAL, 
                   sd, 
                   slot->index,
                   slot->index_remote,
@@ -491,8 +546,9 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
                   session->test_duration,
                   session->total_reads,
                   session->total_writes,
+		  slot->seq_highest_recv,
                   slot->total_pkt_lost,
-                  slot->total_pkt_dup);
+                  slot->total_pkt_ooo);
 
   session->byte_total_recv = 0;
   session->byte_total_sent = 0;
@@ -500,6 +556,11 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
   session->total_writes    = 0;
   session->duration_done   = 1;
 
+  slot->subtotal_pkt_lost = 0;
+  slot->total_pkt_lost    = 0;
+  slot->total_pkt_recv    = 0;
+  slot->total_pkt_ooo     = 0;
+
   udp_slot_kill(slot);
 
   return OOP_CONTINUE;
@@ -969,6 +1030,11 @@ static void udp_slot_kill(nepim_slot_t *slot)
 
   cancel_slot_segment_read(slot);
 
+  nepim_global.oop_src->cancel_signal(nepim_global.oop_src,
+				      SIGINT, on_sigint, slot);
+  nepim_global.oop_src->cancel_signal(nepim_global.oop_src,
+				      SIGTERM, on_sigterm, slot);
+
   nepim_slot_set_del(&slots, slot->index);
 }
 
