@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_server.c,v 1.83 2006/04/24 14:59:18 evertonm Exp $ */
+/* $Id: udp_server.c,v 1.91 2006/05/11 12:23:35 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -64,7 +64,12 @@ static void *on_udp_interval(oop_source *src, struct timeval tv, void *user)
   nepim_slot_t *slot = user;
   nepim_session_t *session = &slot->session;
 
-  nepim_slot_stat(stdout, NEPIM_LABEL_PARTIAL, 
+  assert(timercmp(&tv, &session->tv_interval, ==));
+
+  nepim_slot_update_pkt_stat(slot);
+
+  nepim_slot_stat(stdout, tv,
+		  NEPIM_LABEL_PARTIAL, 
 		  slot->udp_sd,
 		  slot->index,
 		  slot->index_remote,
@@ -75,18 +80,20 @@ static void *on_udp_interval(oop_source *src, struct timeval tv, void *user)
 		  session->test_duration,
 		  session->interval_reads,
 		  session->interval_writes,
+		  slot->seq_highest_recv - slot->interval_last_highest_seq,
 		  slot->interval_pkt_lost,
-		  slot->interval_pkt_dup);
+		  slot->interval_pkt_ooo);
 
   session->byte_interval_recv = 0;
   session->byte_interval_sent = 0;
   session->interval_reads     = 0;
   session->interval_writes    = 0;
 
-  slot->total_pkt_lost += slot->interval_pkt_lost;
-  slot->total_pkt_dup += slot->interval_pkt_dup;
+  slot->interval_last_highest_seq = slot->seq_highest_recv;
+
+  slot->interval_pkt_recv = 0;
   slot->interval_pkt_lost = 0;
-  slot->interval_pkt_dup = 0;
+  slot->interval_pkt_ooo  = 0;
 
   schedule_stat_interval(slot->index);
 
@@ -119,10 +126,12 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &slot->session;
   int sd = slot->udp_sd;
 
-  slot->total_pkt_lost += slot->interval_pkt_lost;
-  slot->total_pkt_dup += slot->interval_pkt_dup;
+  assert(timercmp(&tv, &session->tv_duration, ==));
 
-  nepim_slot_stat(stdout, NEPIM_LABEL_TOTAL, 
+  nepim_slot_update_pkt_stat(slot);
+
+  nepim_slot_stat(stdout, tv,
+		  NEPIM_LABEL_TOTAL, 
 		  sd, 
 		  slot->index,
 		  slot->index_remote,
@@ -133,8 +142,9 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
 		  session->test_duration,
 		  session->total_reads,
 		  session->total_writes,
+		  slot->seq_highest_recv,
 		  slot->total_pkt_lost,
-		  slot->total_pkt_dup);
+		  slot->total_pkt_ooo);
 
   session->byte_total_recv = 0;
   session->byte_total_sent = 0;
@@ -142,6 +152,11 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
   session->total_writes    = 0;
   session->duration_done   = 1;
 
+  slot->subtotal_pkt_lost = 0;
+  slot->total_pkt_recv    = 0;
+  slot->total_pkt_lost    = 0;
+  slot->total_pkt_ooo     = 0;
+
 #if 0
   udp_slot_cancel_stat_timers(slot);
   udp_slot_cancel_writers(slot);
@@ -231,6 +246,11 @@ static void udp_slot_kill(nepim_slot_t *slot)
   nepim_slot_set_del(&slots, slot_index);
 }
 
+void nepim_slot_kill(nepim_slot_t *slot)
+{
+  udp_slot_kill(slot);
+}
+
 static void will_slot_rate_write(nepim_slot_t *slot)
 {
   nepim_session_t *session = &slot->session;
@@ -588,7 +608,8 @@ static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *
 	    "%d %d-%d: broken slot: peer not sending keepalives?\n",
 	    slot->udp_sd, slot->index, slot->index_remote);
 
-    report_broken_slot_stat(stdout, slot);
+    if (!session->duration_done)
+      report_broken_slot_stat(stdout, slot);
 
     udp_slot_kill(slot);
 
@@ -787,18 +808,18 @@ static int parse_greetings(nepim_greet_t *opt,
   return 0;
 }
 
-static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
+static void udp_consume(nepim_slot_t *slot, size_t len,
+			const nepim_udp_hdr_t *hdr)
 {
   nepim_session_t *session = &slot->session;
 
   /* any valid traffic accounted as keepalive */
   ++session->keepalives_recv;
 
-  nepim_session_read_add(session, len + session->overhead);
+  if (hdr->type == UDP_TYPE_DATA)
+    nepim_slot_seq_recv(slot, hdr->seq);
 
-  if (nepim_global.udp_exp_stats)
-    nepim_win_add(&slot->win_recv_seq, seq,
-		  &slot->interval_pkt_lost, &slot->interval_pkt_dup);
+  nepim_session_read_add(session, len + session->overhead);
 }
 
 static void *on_udp_segment(oop_source *src, int sd,
@@ -866,7 +887,7 @@ static void *on_udp_segment(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  if (hdr.version) {
+  if (hdr.version != UDP_VERSION) {
     fprintf(stderr, "%d: recvfrom: ignoring unknown UDP header version: %d\n",
 	    sd, hdr.version);
     return OOP_CONTINUE;
@@ -887,6 +908,12 @@ static void *on_udp_segment(oop_source *src, int sd,
 	    nepim_uint16_read(buf + 3));
 #endif
 
+    if (hdr.seq) {
+      fprintf(stderr, "%d %d-%d: warn: ignoring invalid UDP hello header sequence: %llu\n",
+	      sd, hdr.dst_slot, hdr.src_slot, hdr.seq);
+      return OOP_CONTINUE;
+    }
+
     if (nepim_slot_find_addr(&slots, hdr.src_slot, 
 			     (const struct sockaddr *) &from,
 			     fromlen)) {
@@ -990,7 +1017,7 @@ static void *on_udp_segment(oop_source *src, int sd,
       return OOP_CONTINUE;
     }
 
-    udp_consume(slot, rd, hdr.seq);
+    udp_consume(slot, rd, &hdr);
 
     udp_check_packet_data(sd, &hdr, &(slot->session), buf, UDP_HEADER_LEN, rd);
   }
